@@ -1,14 +1,12 @@
-.rx <- loadNamespace("rxode2")
-
 transTo <- function(model, syntax, match = TRUE) {
   mv <- rxModelVars(model)
   if (match) {
     test_that(sprintf("%s includes %s", model, syntax), {
-      expect_true(regexpr(syntax, .rx$.rxGetParseModel(), fixed = TRUE) != -1)
+      expect_true(regexpr(syntax, .rxGetParseModel(), fixed = TRUE) != -1)
     })
   } else {
     test_that(sprintf("%s dose not include %s", model, syntax), {
-      expect_false(regexpr(syntax, .rx$.rxGetParseModel(), fixed = TRUE) != -1)
+      expect_false(regexpr(syntax, .rxGetParseModel(), fixed = TRUE) != -1)
     })
   }
 }
