@@ -1,17 +1,12 @@
 rxTest({
   test_that("rxStack preserves factors", {
-
-    .rx <- loadNamespace("rxode2")
-
     pheno2 <- function() {
       ini({
-        tcl <- log(0.008) # typical value of clearance
-        tv <-  log(0.6)   # typical value of volume
-        ## var(eta.cl)
+        tcl <- log(0.008)
+        tv <-  log(0.6)
         eta.cl + eta.v ~ c(1,
-                           0.01, 1) ## cov(eta.cl, eta.v), var(eta.v)
-        # interindividual variability on clearance and volume
-        add.err <- 0.1    # residual variability
+                           0.01, 1)
+        add.err <- 0.1
       })
       model({
         cl <- exp(tcl + eta.cl) # individual value of clearance
@@ -25,9 +20,8 @@ rxTest({
 
     simdata <- data.frame(time=1:10, ID=factor(c("A", "B")))
 
-    sim <- .rx$rxSolve(pheno2, events=simdata)
+    sim <- rxSolve(pheno2, events=simdata)
 
     expect_equal(levels(sim$id), c("A", "B"))
-
   })
 })
